Package: Countr
Type: Package
Title: Flexible Univariate Count Models Based on Renewal Processes
Version: 3.6.1
Authors@R: c( person(given = "Tarak", family = "Kharrat",
                     role = c("aut"), 
                     email = "tarak@realanalytics.co.uk",
		     comment = c(ORCID = "0000-0001-9399-6174")),
              person(given = c("Georgi", "N."), family = "Boshnakov", 
                     role = c("aut", "cre"), 
                     email = "georgi.boshnakov@manchester.ac.uk",
		     comment = c(ORCID = "0000-0003-2839-346X")) )
Description: Flexible univariate count models based on renewal processes. The
    models may include covariates and can be specified with familiar formula
    syntax as in glm() and package 'flexsurv'.  The methodology is described by
    Kharrat et all (2019) <doi:10.18637/jss.v090.i13> (included as vignette
    'Countr_guide' in the package).
License: GPL (>= 2)
URL: https://geobosh.github.io/Countr/ (doc),
        https://CRAN.R-project.org/package=Countr
BugReports: https://github.com/GeoBosh/Countr/issues
Depends: R (>= 3.3.0)
Imports: Rcpp (>= 0.11.3), flexsurv, Formula, VGAM (>= 1.1-1), optimx,
        numDeriv, boot, MASS, utils, Rdpack (>= 0.7-0), dplyr,
        standardize, pscl, car, Matrix
LinkingTo: Rcpp, RcppArmadillo
Suggests: testthat, knitr, xtable, lmtest, lattice, RColorBrewer
RdMacros: Rdpack
VignetteBuilder: knitr
LazyData: true
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Collate: 'Countr-package.R' 'RcppExports.R' 'anc.R' 'coefnames.R'
        'convCount_loglik.R' 'convCount_moments.R' 'convCount_probs.R'
        'dBivariateWeibull.R' 'dWeibull.R' 'dWeibullgamma.R' 'data.R'
        'renewal_IV.R' 'renewal_tools.R' 'renewal_cstr.R' 'tools.R'
        'renewal_methods.R'
Config/build/clean-inst-doc: FALSE
Packaged: 2026-01-18 14:57:20 UTC; georgi
Author: Tarak Kharrat [aut] (ORCID: <https://orcid.org/0000-0001-9399-6174>),
  Georgi N. Boshnakov [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2839-346X>)
Maintainer: Georgi N. Boshnakov <georgi.boshnakov@manchester.ac.uk>
Repository: CRAN
Date/Publication: 2026-01-18 15:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 14:03:52 UTC; unix
Archs: Countr.so.dSYM
