# Generated by roxygen2: do not edit by hand

S3method(addBootSampleObject,renewal)
S3method(chiSq_gof,glm)
S3method(chiSq_gof,negbin)
S3method(chiSq_gof,renewal)
S3method(chiSq_pearson,glm)
S3method(chiSq_pearson,renewal)
S3method(coef,renewal)
S3method(confint,renewal)
S3method(df.residual,renewal)
S3method(extractAIC,renewal)
S3method(fitted,renewal)
S3method(logLik,renewal)
S3method(model.matrix,renewal)
S3method(nobs,renewal)
S3method(predict,modelMat)
S3method(predict,renewal)
S3method(print,renewal)
S3method(print,summary.renewal)
S3method(renewalCoef,default)
S3method(renewalCoef,glm)
S3method(renewalCoef,renewalCoefList)
S3method(renewalNames,character)
S3method(renewalNames,default)
S3method(renewalNames,formula)
S3method(residuals,renewal)
S3method(residuals_plot,renewal)
S3method(se.coef,renewal)
S3method(summary,renewal)
S3method(update,renewal)
S3method(vcov,renewal)
export(CountrFormula)
export(addBootSampleObject)
export(chiSq_gof)
export(chiSq_pearson)
export(compareToGLM)
export(count_table)
export(dBivariateWeibullCountFrankCopula)
export(dBivariateWeibullCountFrankCopula_loglik)
export(dCount_conv_bi)
export(dCount_conv_loglik_bi)
export(dCount_conv_loglik_user)
export(dCount_conv_user)
export(dRenewalFrankCopula_bi)
export(dRenewalFrankCopula_user)
export(dWeibullCount)
export(dWeibullCount_loglik)
export(dWeibullgammaCount)
export(dWeibullgammaCount_loglik)
export(dmodifiedCount_bi)
export(evCount_conv_bi)
export(evCount_conv_user)
export(evWeibullCount)
export(evWeibullgammaCount)
export(frequency_plot)
export(getParNames)
export(renewal)
export(renewal.control)
export(renewal.convPars)
export(renewal.seriesPars)
export(renewal.weiMethod)
export(renewalCoef)
export(renewalCoefList)
export(renewalCount)
export(renewalNames)
export(residuals_plot)
export(se.coef)
export(surv)
import(Formula)
import(Rcpp)
import(flexsurv)
import(optimx)
importFrom(MASS,ginv)
importFrom(Rdpack,reprompt)
importFrom(VGAM,cauchitlink)
importFrom(VGAM,clogloglink)
importFrom(VGAM,logitlink)
importFrom(VGAM,probitlink)
importFrom(boot,boot)
importFrom(car,Boot)
importFrom(dplyr,contains)
importFrom(dplyr,groups)
importFrom(dplyr,left_join)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(numDeriv,hessian)
importFrom(numDeriv,jacobian)
importFrom(pscl,predprob)
importFrom(standardize,standardize)
importFrom(stats,AIC)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,confint.default)
importFrom(stats,density)
importFrom(stats,dnbinom)
importFrom(stats,dpois)
importFrom(stats,family)
importFrom(stats,formula)
importFrom(stats,getCall)
importFrom(stats,glm.fit)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,na.pass)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,terms.formula)
importFrom(stats,update)
importFrom(stats,update.formula)
importFrom(stats,vcov)
importFrom(utils,capture.output)
useDynLib(Countr)
