<!-- badges: start -->
[![CRANStatusBadge](http://www.r-pkg.org/badges/version/Countr)](https://cran.r-project.org/package=Countr)
[![R-CMD-check](https://github.com/GeoBosh/Countr/workflows/R-CMD-check/badge.svg)](https://github.com/GeoBosh/Countr/actions)
<!-- badges: end -->

# Installing Countr

The [latest stable version](https://cran.r-project.org/package=Countr) is on
CRAN.

    install.packages("Countr")

You can install the [development version](https://github.com/GeoBosh/Countr) of
`Countr` from Github:

    remotes::install_github("GeoBosh/Countr")


# Overview

Flexible univariate count models based on renewal processes. The models may
include covariates and can be specified with familiar formula syntax as in `glm()`
and package 'flexsurv'.  The methodology is described in a paper in
the Journal of Statistical Software. The paper is included as vignette
`vignette("Countr_guide")` along wth several additional vignettes containing
worked examples and additional details, see
[Countr CRAN page](https://cran.r-project.org/package=Countr).

The package is developed by Tarak Kharrat and Georgi N.Boshnakov.
